#!/usr/bin/env python
# -*- coding: utf-8; buffer-read-only: t -*-

__author__ = "Gregorio Ambrosio"
__contact__ = "gambrosio[at]uma.es"
__copyright__ = "Copyright 2021, Gregorio Ambrosio"
__date__ = "2021/07/27"
__license__ = "MIT"

import numpy as np
import cv2  # note: pip install opencv-python opencv-contrib-python
import pandas as pd
import matplotlib.patches as mpatches
import matplotlib.pyplot as plt

from robotathome import logger


__all__ = ['get_labeled_img',
           'plot_labeled_img',
           'get_scan_xy',
           'plot_scan',
           'plot_scene',
           'process_image',
           'concat_images']


# computer vision related functions


def get_labeled_img(labels, img_file):
    """
    Return an image patched with labels.

    Input
    =====
    labels:
        Label dataframe with masks (label df + mask col <- get_label_masks(id))

    img-file:
        Full path filename of the RGB image

    Output
    ======
    labeled_image:
        RGB image patched with colored labels. Labels are colored and
        over imposed with transparency
    colors:
        A list with the RGB + Alpha components (4 x 0<= n <=1) of randomly
        chosen colors
    """
    alpha = 0.7
    bgr_img = cv2.imread(img_file, cv2.IMREAD_COLOR)
    labeled_img, colors = overlay_mask(cv2.cvtColor(bgr_img, cv2.COLOR_BGR2RGB),
                                       labels['mask'],
                                       alpha)
    return [labeled_img, colors]


def plot_labeled_img(labels, img_file):
    """
    Plot a RGB image patched with colored labels with legend.

    Input
    =====
    labels:
        Label dataframe with masks (label df + mask col <- get_label_masks(id))

    img-file:
        Full path filename of the RGB image

    """
    [labeled_img, colors] = get_labeled_img(labels, img_file)
    plot_mask(labeled_img, labels['name'], colors)


def get_scan_xy(laser_scan):

    # Computing the smallest angle change
    da = laser_scan.aperture / laser_scan.no_of_shots
    # epsilon = np.finfo(np.float64).eps

    # Computing x, y coodinates
    dist  = laser_scan['scan']
    angle = laser_scan['shot_id']*da
    valid = laser_scan['valid_scan']

    # x, y will be 0 when not valid
    x = dist * np.cos(angle) * valid
    y = dist * np.sin(angle) * valid
    x.name = 'x'
    y.name = 'y'
    xy = pd.concat([x, y], axis=1)
    # xy(columns = ['x', 'y'])
    xy.index.name = 'shot_id'
    return xy


def plot_scan(laser_scan, cmap='gist_heat'):

    # Getting a laser scan dataframe with only valid values
    valid_laser_scan = laser_scan.loc[laser_scan['valid_scan'] == 1]

    # Computing the smallest angle change
    da = laser_scan.aperture / laser_scan.no_of_shots
    # epsilon = np.finfo(np.float64).eps

    # Computing x, y coodinates
    dist  = valid_laser_scan['scan']
    angle = valid_laser_scan['shot_id']*da

    x = dist * np.cos(angle)
    y = dist * np.sin(angle)

    # Plotting points
    # plt.plot(x, y, marker=".", markersize=2)
    sc = plt.scatter(x, y, vmin=0, vmax=laser_scan.max_range, c=dist, s=1, cmap = cmap) # or 'gist_heat'
    plt.scatter(0, 0, s=50)
    plt.colorbar(sc, label='Distance (m)')
    plt.show()


def plot_scene(scene_file):
    """
    Plot a scene using open3d package.

    Input
    =====
    scene_file:
        File containing the point cloud
    """
    # https://towardsdatascience.com/guide-to-real-time-visualisation-of-massive-3d-point-clouds-in-python-ea6f00241ee0
    # https://towardsdatascience.com/discover-3d-point-cloud-processing-with-python-6112d9ee38e7

    # from mpl_toolkits import mplot3d
    # point_cloud = np.loadtxt(scene_file, skiprows=6)
    # xyz = point_cloud[:,:3]
    # rgb = point_cloud[:,3:]
    # ax = plt.axes(projection='3d')
    # ax.scatter(xyz[:,0], xyz[:,1], xyz[:,2], c = rgb, s=0.01)
    # plt.show()

    # point_cloud = np.loadtxt(scene_file, skiprows=6)
    # xyz = point_cloud[:,:3]
    # rgb = point_cloud[:,3:]
    # pcd = o3d.geometry.PointCloud()
    # pcd.points = o3d.utility.Vector3dVector(xyz)
    # pcd.colors = o3d.utility.Vector3dVector(rgb)
    # o3d.visualization.draw_geometries([pcd])

    import open3d as o3d
    pcd = o3d.io.read_point_cloud(scene_file, format='xyzrgb')
    # downpcd = pcd.voxel_down_sample(voxel_size=0.01)
    o3d.visualization.draw_geometries([pcd])


# Related to RGBD generic function.


def process_image(f, img):
    """ Apply generic function f over img."""
    return f(img)


def concat_images(img_dict):  # (img, par1, ... parn)
    """ Process the image. """
    # do something with the img
    img_list = list(img_dict.values())
    composed_img = cv2.hconcat(img_list)
    return composed_img


def RGBD_stiching(img_dict):  # (img, par1, ... parn)
    """ Process the image. """
    # do something with the img
    img_list = list(img_dict.values())
    stitcher = cv2.Stitcher.create(cv2.Stitcher_PANORAMA)
    status, pano = stitcher.stitch(img_list)
    if status != cv2.Stitcher_OK:
        # error code = 1 means that there are no enough features between the 
        # image intersections.
        logger.error("Can't stitch images, error code = %d" % status)
        sys.exit(-1)
    return pano


# Helpers


def bin2rgba(img):
    """
    TODO
    """

    img = cv2.cvtColor(img*255, cv2.COLOR_GRAY2RGB)
    color_mask = np.random.random((1, 3)).tolist()[0]

    my_mask = cv2.compare(img, 245, cv2.CMP_GT)
    img[my_mask > 0] = 255

    for i in range(3):
        img[:, :, i] = color_mask[i]*255
    plt.imshow(img, interpolation='nearest')
    plt.show()


def overlay_mask(img, masks, alpha=0.5):
    # cv2.addWeighted(ovl_img, alpha, base_img, 1 - alpha, 0, base_img)
    # return base_img
    colors = []
    for mask in masks:
        color = np.random.random(3)
        colors.append(np.append(color, [alpha]))
        mask = np.repeat((mask > 0)[:, :, np.newaxis], repeats=3, axis=2)
        img = np.where(mask, img * (1 - alpha) + color*255 * alpha, img)
    return img.astype('uint8'), colors


def plot_mask(patched_img, names, colors):
    plt.imshow(patched_img)
    plt.xticks([]), plt.yticks([])  # to hide tick values on X and Y axis
    mpatches_ = []
    for i, color_ in enumerate(colors):
        mpatches_.append(mpatches.Patch(color=color_, label=names[i]))
    plt.legend(handles=mpatches_)
    plt.show()


# Lab


# Old stuff


def get_video_from_rgbd(self,
                        source='lblrgbd',
                        home_session_name='alma-s1',
                        home_subsession=0,
                        room_name='alma_masterroom1',
                        sensor_name='RGBD_1',
                        video_file_name=None
                        ):
    """
    This functions ...
    """
    rows = self.get_sensor_observation_files(source,
                                             home_session_name,
                                             home_subsession,
                                             room_name,
                                             sensor_name)

    # Computing frames per second
    num_of_frames = len(rows)
    seconds = (rows.iloc[-1]['t'] - rows.iloc[0]['t']) / 10**7
    frames_per_second = num_of_frames / seconds
    rh.logger.debug("frames per second: {:.2f}", frames_per_second)

    # Get frame size
    image_path = rows.iloc[0]['pth']
    file_name = rows.iloc[0]['f2']
    image_path_file_name = os.path.join(self.__rh_path,
                                        self.__rgbd_path,
                                        image_path,
                                        file_name)
    img = cv2.imread(image_path_file_name, cv2.IMREAD_COLOR)
    img_h, img_w, _ = img.shape

    # Opening video file
    if video_file_name is None:
        video_file_name = ''.join(
            [
                home_session_name,
                '_', str(home_subsession),
                '_', room_name,
                '_', sensor_name,
                dt.datetime.now().strftime("_%Y%m%d%H%M%S"),
                '.avi'
            ]
        )
    fourcc = cv2.VideoWriter_fourcc(*'MJPG')
    video_path_file_name = os.path.abspath(os.path.join(self.__wspc_path,
                                                        video_file_name)
                                           )
    out = cv2.VideoWriter(video_path_file_name,
                          fourcc,
                          frames_per_second,
                          (img_w, img_h))

    for _, row in rows.iterrows():
        image_path = row['pth']
        file_name = row['f2']
        image_path_file_name = os.path.join(self.__rh_path,
                                            self.__rgbd_path,
                                            image_path,
                                            file_name)
        img = cv2.imread(image_path_file_name, cv2.IMREAD_COLOR)

        if rh.is_being_logged():
            cv2.imshow('Debug mode (press q to exit)', img)
            if cv2.waitKey(1) & 0xFF == ord('q'):
                break

        out.write(img)

    out.release()

    if rh.is_being_logged():
        cv2.destroyAllWindows()

    return video_file_name


def get_composed_video_from_lblrgbd(self,
                                    home_session_name='alma-s1',
                                    home_subsession=0,
                                    room_name='alma_masterroom1',
                                    video_file_name=None
                                    ):

    ''' Docstring '''

    rows_rgbd_1 = self.get_sensor_observation_files('lblrgbd',
                                                    home_session_name,
                                                    home_subsession,
                                                    room_name,
                                                    'RGBD_1')

    rows_rgbd_2 = self.get_sensor_observation_files('lblrgbd',
                                                    home_session_name,
                                                    home_subsession,
                                                    room_name,
                                                    'RGBD_2')

    rows_rgbd_3 = self.get_sensor_observation_files('lblrgbd',
                                                    home_session_name,
                                                    home_subsession,
                                                    room_name,
                                                    'RGBD_3')

    rows_rgbd_4 = self.get_sensor_observation_files('lblrgbd',
                                                    home_session_name,
                                                    home_subsession,
                                                    room_name,
                                                    'RGBD_4')

    # Computing frames per second
    num_of_frames = len(rows_rgbd_1)
    seconds = (rows_rgbd_1.iloc[-1]['t'] - rows_rgbd_1.iloc[0]['t']) / 10**7
    frames_per_second = num_of_frames / seconds
    rh.logger.debug("frames per second: {:.2f}", frames_per_second)

    # Get frame size
    image_path = rows_rgbd_1.iloc[0]['pth']
    file_name = rows_rgbd_1.iloc[0]['f2']
    image_path_file_name = os.path.join(self.__rh_path,
                                        self.__rgbd_path,
                                        image_path,
                                        file_name)
    img = cv2.imread(image_path_file_name, cv2.IMREAD_COLOR)
    img_h, img_w, _ = img.shape

    # Opening video file
    if video_file_name is None:
        video_file_name = ''.join(
            [
                home_session_name,
                '_', str(home_subsession),
                '_', room_name,
                '_RGBD_3412',
                dt.datetime.now().strftime("_%Y%m%d%H%M%S"),
                '.avi'
            ]
        )

    fourcc = cv2.VideoWriter_fourcc(*'MJPG')
    video_path_file_name = os.path.abspath(os.path.join(self.__wspc_path,
                                                        video_file_name))
    out = cv2.VideoWriter(video_path_file_name,
                          fourcc,
                          frames_per_second,
                          (4 * img_w, img_h))

    for i in range(len(rows_rgbd_1)):
        image_rgbd_1_path = rows_rgbd_1.iloc[i, 2]
        file_rgbd_1_name = rows_rgbd_1.iloc[i, 4]
        image_rgbd_1_path_file_name = os.path.join(self.__rh_path,
                                                   self.__rgbd_path,
                                                   image_rgbd_1_path,
                                                   file_rgbd_1_name)

        image_rgbd_2_path = rows_rgbd_2.iloc[i, 2]
        file_rgbd_2_name = rows_rgbd_2.iloc[i, 4]
        image_rgbd_2_path_file_name = os.path.join(self.__rh_path,
                                                   self.__rgbd_path,
                                                   image_rgbd_2_path,
                                                   file_rgbd_2_name)

        image_rgbd_3_path = rows_rgbd_3.iloc[i, 2]
        file_rgbd_3_name = rows_rgbd_3.iloc[i, 4]
        image_rgbd_3_path_file_name = os.path.join(self.__rh_path,
                                                   self.__rgbd_path,
                                                   image_rgbd_3_path,
                                                   file_rgbd_3_name)

        image_rgbd_4_path = rows_rgbd_4.iloc[i, 2]
        file_rgbd_4_name = rows_rgbd_4.iloc[i, 4]
        image_rgbd_4_path_file_name = os.path.join(self.__rh_path,
                                                   self.__rgbd_path,
                                                   image_rgbd_4_path,
                                                   file_rgbd_4_name)


        img_rgbd_1 = cv2.imread(image_rgbd_1_path_file_name, cv2.IMREAD_COLOR)
        img_rgbd_2 = cv2.imread(image_rgbd_2_path_file_name, cv2.IMREAD_COLOR)
        img_rgbd_3 = cv2.imread(image_rgbd_3_path_file_name, cv2.IMREAD_COLOR)
        img_rgbd_4 = cv2.imread(image_rgbd_4_path_file_name, cv2.IMREAD_COLOR)

        img = cv2.hconcat([img_rgbd_3, img_rgbd_4, img_rgbd_1, img_rgbd_2])

        if rh.is_being_logged():
            cv2.imshow('Debug mode (press q to exit)', img)
            if cv2.waitKey(1) & 0xFF == ord('q'):
                break

        out.write(img)
    out.release()

    if rh.is_being_logged():
        cv2.destroyAllWindows()

    return video_file_name


def get_rgb_image_from_lblrgbd(self, so_id):
    """
    This function 

    Parameters
    ----------

    Returns
    -------
    A BGR cv2 image

    """

    sql_str = f"""
    select pth, f2
    from rh_temp_lblrgbd
    where id = {so_id}
    """

    df_rows = pd.read_sql_query(sql_str, self.__con)
    # rh.logger.debug("df_rows.shape: {}", df_rows.shape)
    rgb_image_path_file_name = os.path.join(self.__rh_path,
                                        self.__rgbd_path,
                                        df_rows.loc[0, "pth"],
                                        df_rows.loc[0, "f2"])

    # rh.logger.debug("rgb_image_path_file_name: {}",
    #                 rgb_image_path_file_name)
    bgr_img = cv2.imread(rgb_image_path_file_name, cv2.IMREAD_COLOR)

    return bgr_img


def get_depth_image_from_lblrgbd(self, so_id):
    """
    This function

    Parameters
    ----------

    Returns
    -------
    A gray levels cv2 image

    """

    sql_str = f"""
    select pth, f1
    from rh_temp_lblrgbd
    where id = {so_id}
    """

    df_rows = pd.read_sql_query(sql_str, self.__con)
    rh.logger.debug("df_rows.shape: {}", df_rows.shape)
    rgb_image_path_file_name = os.path.join(self.__rh_path,
                                        self.__rgbd_path,
                                        df_rows.loc[0, "pth"],
                                        df_rows.loc[0, "f1"])

    rh.logger.debug("rgb_image_path_file_name: {}",
                    rgb_image_path_file_name)
    img = cv2.imread(rgb_image_path_file_name, cv2.IMREAD_COLOR)

    return cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)


def lblrgbd_plot_labels(self, so_id):
    img = self.get_rgb_image_from_lblrgbd(so_id)
    labels = self.get_labels_from_lblrgbd(so_id)
    rh.logger.debug("labels: {}", labels)
    mask = self.get_mask_from_lblrgbd(so_id)
    label_mask = self.get_label_mask(mask, labels['local_id'])
    alpha = 0.7
    out_img, colors = rh.overlay_mask(cv2.cvtColor(img, cv2.COLOR_BGR2RGB),
                                      label_mask,
                                      alpha)
    rh.plot_mask(out_img, labels['name'], colors)



#!/usr/bin/env python
# -*- coding: utf-8; buffer-read-only: t -*-

__version__ = "1.1.8"

from ._version import *
from .log import *
from .helpers import *

# from . import cv # rh.cv.say_hello
# from .cv import say_hello, hello_cv # rh.say_hello
from .cv import *
from .core import *

__all__ = []
__all__.extend(_version.__all__)
__all__.extend(log.__all__)
__all__.extend(helpers.__all__)
__all__.extend(core.__all__)

set_log_level('SUCCESS')
# set_log_level('DEBUG')



#!/usr/bin/env python
# -*- coding: utf-8; buffer-read-only: t -*-

__author__ = "Gregorio Ambrosio"
__contact__ = "gambrosio[at]uma.es"
__copyright__ = "Copyright 2021, Gregorio Ambrosio"
__date__ = "2021/02/22"
__license__ = "MIT"

import os
import datetime as dt
import time
import re
import hashlib
import tarfile
import io
import sys
import requests
from tqdm.auto import tqdm
from .log import logger

__all__ = ['download', 'get_md5', 'uncompress',
           'flat2Dlist', 'flatlist', 'reverse_dict',
           'time_win2unixepoch', 'time_unixepoch2win',
           'rename_if_exist'
           ]

"""
Helper functions
"""

# Download related

def download(url: str, path: str = os.getcwd()) -> None:
    """
    Download file with progressbar

    Args:
        url: a hyperlink that points to a location where the file to download
    resides.
        path: path where the file will be stored

    Example:
        >>> import robotathome as rh
        >>> rh.download('https://...','~/Download')
    """
    # if not filename:
    #     local_filename = os.path.join(".", url.split('/')[-1])
    # else:
    #     local_filename = filename

    # print("Unexpected error: ",sys.exc_info()[0], " occurred.")

    req = requests.get(url, stream=True)
    logger.debug("Status code: {}", req.status_code)

    try:
        req.raise_for_status()
    except requests.exceptions.RequestException as error:
        logger.info("The file couldn\'t be retrieved")
        logger.error("Error: {}", error)
        # Whoops it wasn't a 200
        # return "Error: " + error
    else:
        if (os.path.isdir(os.path.expanduser(path))):
            remote_filename = req.headers.get("Content-Disposition").split("filename=")[1]
            logger.debug("remote_filename: {}", remote_filename)
            local_filename = os.path.expanduser(os.path.join(path, remote_filename.strip(' " " ')))
            logger.debug("local_filename: {}", local_filename)
            file_size = int(req.headers['Content-Length'])
            chunk = 1
            chunk_size = 2**20 # 1024 for KB, 1024*1024 for MB
            num_bars = int(file_size / chunk_size)
            try:
                with open(local_filename, 'wb') as fp:
                    for chunk in tqdm(req.iter_content(chunk_size=chunk_size),
                                      total=num_bars,
                                      unit='MB',
                                      # unit_scale=1,
                                      # unit_divisor=2**10,
                                      desc=local_filename,
                                      leave=True  # progressbar stays
                                      ):
                        fp.write(chunk)
                logger.info('Sucessfully downloaded: {}', local_filename)
            except Exception as error:
                logger.error("Error: {}", error)
                logger.info('Something went wrong trying to download {}', local_filename)
            finally:
                fp.close()

        else:
            logger.error("Error: The directory {} doesn\'t exist", path)
            logger.info("The file couldn\'t be retrieved")

def get_md5(filename: str) -> str:
    """Computes MD5 hash of a given file

    Args:
        filename: the filename to get the md5 hash

        path: the path where the filename is located

    Returns:
        a string with the MD5 hash value
    """

    try:
        local_filename = os.path.expanduser(filename)
        chunk_size = 65536
        hasher = hashlib.md5()
        with open(local_filename, 'rb') as afile:
            buf = afile.read(chunk_size)
            while len(buf) > 0:
                hasher.update(buf)
                buf = afile.read(chunk_size)
        logger.debug("MD5 checksum for {} : {}",
                        local_filename,
                        hasher.hexdigest()
                        )
        return hasher.hexdigest()
    except Exception as error:
        logger.error("Error: {}", error)
        return ''

def uncompress(filename: str, path: str = os.getcwd()) -> None:
    """Uncompress a tar file

       Args:
           filename: a tar file (tar, tgz, ...)
           path: where the filename will be uncompressed

    Example:
        >>> import robotathome as rh
        >>> rh.uncompress('~/WORKSPACE/Robot@Home2_db.tgz')
    """
    class ProgressFileObject(io.FileIO):
        def __init__(self, path, *args, **kwargs):
            self._total_size = os.path.getsize(path)
            io.FileIO.__init__(self, path, *args, **kwargs)

        def read(self, size):
            sys.stdout.write("\rUncompressing %d of %d MB (%d%%)" % (self.tell() / 1048576, self._total_size / 1048576, self.tell()*100/self._total_size))
            sys.stdout.flush()
            return io.FileIO.read(self, size)

    try:
        logger.info("Extracting files from {}: ", (os.path.basename(filename)))
        file_obj=ProgressFileObject(os.path.expanduser(filename))
        tf = tarfile.open(fileobj=file_obj)
        tf.extractall(path=os.path.expanduser(path))
        file_obj.close()
    except Exception as error_code:
        logger.info("Error: {}", error_code)
    else:
        tf.close()
        print()
        logger.info("Extraction success. Don't forget to remove {} if you are not plenty of space.",
                       (os.path.basename(filename)))


# Misc

def flat2Dlist(list_):
    return sum(list_, [])

def flatlist(list_):
    if len(list_) == 0:
        return list_
    if isinstance(list_[0], list):
        return flatlist(list_[0]) + flatlist(list_[1:])
    return list_[:1] + flatlist(list_[1:])

def reverse_dict(dict_):
    return dict(map(reversed, dict_.items()))


# Time

def time_win2unixepoch(time_stamp):
    ''' Doctring '''
    seconds = time_stamp / 10000000
    epoch = seconds - 11644473600
    datetime_ = dt.datetime(2000, 1, 1, 0, 0, 0)
    return datetime_.fromtimestamp(epoch)

def time_unixepoch2win(date):
    ''' Doctring '''
    match = re.compile(r'^(\d{4})-(\d\d)-(\d\d)T(\d\d):(\d\d):(\d\d)$').match(date)
    if match:
        datetime_ = dt.datetime(*map(int, match.groups()))
        windows_timestamp = (time.mktime(datetime_.timetuple()) + 11644473600) * 10000000
    else:
        logger.error("Invalid date format specified: {}\n Specify a date and time string in the format: \"yyyy-MM-ddTHH:mm:ss\"")
        windows_timestamp = 0
    return windows_timestamp

# Log

def rename_if_exist(file_name, tail='.bak'):
    if os.path.isfile(file_name):
        logger.warning("This file name already exists. Adding .bak")
        new_file_name = file_name + tail
        os.rename(file_name, new_file_name)

# Lab



#!/usr/bin/env python
# -*- coding: utf-8; buffer-read-only: t -*-

__author__ = "Gregorio Ambrosio"
__contact__ = "gambrosio[at]uma.es"
__copyright__ = "Copyright 2023, Gregorio Ambrosio"
__date__ = "2023/02/20"
__license__ = "MIT"


import cv2
from robotathome import logger


__all__ = ['filter_sensor_observations',
           'get_frames_per_second',
           'get_RGBD_image',
           'RGBD_images',
           'composed_RGBD_images']


def filter_sensor_observations(rh_dataset, df,
                               home_session_name='alma-s1',
                               home_subsession=0,
                               room_name='alma_masterroom1',
                               sensor_list=['RGBD_3',
                                            'RGBD_4',
                                            'RGBD_1',
                                            'RGBD_2']
                               ):
    """Return a dictionary with an item per sensor.

    This function applies a filter to the dataframe to select data
    for a home session and home subsession, a room name, and a list
    of sensor names.

    Args:
        rh_dataset: a robotathome object to get the id associated to a name
        df: a Pandas dataframe with the data resulting from the
            execution of the RoboAtHome:get_sensor_observations method.
        home_session_name: a string with the home session name
        home_subsession: the subsession number (0 or 1)
        room_name: a string with the room name
        sensor_list: a list made up of the names of the sensors

    Returns:
        df_dict: a dictionary with a item per sensor. Keys are the sensor
        names, and values are the filtered dataframe for that sensor.
    """
    # We need to get some ids from names because
    home_session_id = rh_dataset.name2id(home_session_name, 'hs')
    room_id = rh_dataset.name2id(room_name, 'r')

    qstr1 = f'home_session_id=={home_session_id}'
    qstr2 = f'home_subsession_id=={home_subsession}'
    qstr3 = f'room_id=={room_id}'

    df_dict = {}

    # Apply filter a get the corresponding subset
    for sensor_name in sensor_list:
        sensor_id = rh_dataset.name2id(sensor_name, 's')
        qstr4 = f'sensor_id=={sensor_id}'
        df_query_str = qstr1 + ' & ' + qstr2 + ' & ' + qstr3 + ' & ' + qstr4
        df_dict[sensor_name] = df.query(df_query_str)
    return df_dict


def get_frames_per_second(df):
    """Compute frames per second for a sensor observations dataframe.

    Args:
        df: a Pandas dataframe with the data resulting from the
            execution of the RoboAtHome:get_sensor_observations method.

    Returns:
        frames_per_second: the result of calculating the time between the
        first frame and the last frame divided by the number of frames
    """
    # Computing frames per second
    num_of_frames = len(df.index)
    seconds = (df.iloc[-1]['timestamp'] - df.iloc[0]['timestamp']) / 10**7
    frames_per_second = num_of_frames / seconds
    logger.debug("frames per second: {:.2f}", frames_per_second)
    return frames_per_second


def get_RGBD_image(rh_dataset, row):
    """Return a tuple composed of RGB & D images."""
    [rgb_f, d_f] = rh_dataset.get_RGBD_files(row['id'])
    RGBD_image = (cv2.imread(rgb_f, cv2.IMREAD_COLOR),
                  cv2.imread(d_f,   cv2.IMREAD_COLOR))
    return RGBD_image


def RGBD_images(rh_dataset, df):
    """ Return a generator function of images."""
    for _, row in df.iterrows():
        img = get_RGBD_image(rh_dataset, row)
        yield img


def composed_RGBD_images(rh_dataset, df_dict):
    """ Return a generator function of composed images.

    Returns:
        A dictionary whose keys are sensor names (sensor_list) and values are
        tuples of RGB image list and D image list of the corresponding sensor.

    """
    # Yield over df_dict, i.e. frame by frame
    df_list = list(df_dict.values())  # Get a list of dataframes
    sensor_list = list(df_dict.keys())
    df_lengths = [len(df) for df in df_list]  # Get a list of df's lengths
    number_of_rows = min(df_lengths)  # Select the minumun length

    # For each row we will make a composed image
    for row_index in range(number_of_rows):
        # For each sensor we will append an image to img_list
        RGB_image_list = []  # initialization of img_list
        D_image_list = []
        for df in df_list:
            row = df.iloc[row_index]
            RGBD_image = get_RGBD_image(rh_dataset, row)
            RGB_image_list.append(RGBD_image[0])
            D_image_list.append(RGBD_image[1])
        RGB_image_dict = dict(zip(sensor_list, RGB_image_list))
        D_image_dict = dict(zip(sensor_list, D_image_list))
        yield (RGB_image_dict, D_image_dict)


#!/usr/bin/env python
# -*- coding: utf-8; buffer-read-only: t -*-

"""
Logger related functions for robotathome package
"""

import sys
from loguru import logger
from .helpers import reverse_dict

__all__=['logger', 'set_log_level',
         'log_levels', 'log_level_name', 'log_level_no',
         'get_current_log_level', 'is_being_logged'
         ]



def set_log_level(level='WARNING', sink=sys.stderr):
    logger.remove()
    logger.add(sink=sink, level=level)
    # logger.configure(handlers=[{"sink": sink, "level": level}])

def log_levels():
    log_levels_key_no = {}
    level_values = logger._core.levels.values()
    for level_value in level_values:
        log_levels_key_no[level_value.no] = level_value.name
    log_levels_key_name = reverse_dict(log_levels_key_no)
    return log_levels_key_no, log_levels_key_name

def log_level_name(level_no):
    log_levels_key_no, _ = log_levels()
    return log_levels_key_no[level_no]

def log_level_no(level_name):
    _, log_levels_key_name = log_levels()
    return log_levels_key_name[level_name]

def get_current_log_level():
    level_no = logger._core.min_level
    level_name = log_level_name(level_no)

    return level_no, level_name

def is_being_logged(level_name='DEBUG'):
    """
    This function returns True if the current logging level is under
    'level' value
    """

    current_log_level_no, _ = get_current_log_level()
    return current_log_level_no <= log_level_no(level_name)



#!/usr/bin/env python
# -*- coding: utf-8; buffer-read-only: t -*-

__author__ = "Gregorio Ambrosio"
__contact__ = "gambrosio[at]uma.es"
__copyright__ = "Copyright 2021, Gregorio Ambrosio"
__date__ = "2021/07/27"
__license__ = "MIT"

import sqlite3
import os
import configparser
import numpy as np
import pandas as pd

from robotathome.log import logger

__all__ = ['RobotAtHome']


# @logger.catch
class RobotAtHome():
    """RobotAtHome class with methods for Robot@Home2 dataset.

    The RobotAtHome class encapsulates methods to access the RobotAtHome
    database. <https://doi.org/10.5281/zenodo.4530453>

    Main attributes:
        rh_path (str, optional):
            root path for robotathome database, usually rh.db
        wspc_path (str, optional):
            workspace path where temporary files are stored
        db_filename (str, optional):
            default database name
        rgbd_path (str, optional):
           path that completes rh_path, where rgbd files are stored
        scene_path (str, optional):
           path that completes rh_path, where scene files are stored
    """

    def __init__(self,
                 rh_path='.',
                 rgbd_path='./files/rgbd',
                 scene_path='./files/scene',
                 wspc_path='.',
                 db_filename='rh.db',
                 config_filename='.rh'
                 ):
        """Initialize instance variables.

        There are two ways to get initial values for main attributes:
        with arguments values or by reading a configuration file. This function
        first tries to read a file called .rh . If the file does not exist,
        the attributes get values from the arguments. Additionally, arguments
        can be provided or default values can be obtained.

        Attributes:
            rh_path: path where the database resides.
            rgbd_path: path where rgbd files are located.
            scene_path: path where scene files are located.
            wspc_path: workspace path where the toolbox stores results or
                intermediate data.
            db_filename: file name of the database.

        Raises:
            Exception: If the object cannot be instantiated (usually because
                a database connection could not be done).

        Note 1: The default folder structure is:
         └─── R@H2-2.0.3
              └── files
              │   ├── rgbd
              │   └── scene
              └── rh.db

        Note 2: Example of .rh file
            # This is a default configuration file
            [PATHS]
            rh_path     = $HOME/R@H2-2.0.3
            rgbd_path   = ~/R@H2-2.0.3/files/rgbd
            scene_path  = ~/R@H2-2.0.1/files/scene
            wspc_path   = ~/R@H2-2.0.3
            db_filename = rh.db

            Paths strings can content environmental variables such as $HOME or
            a tilde prefix.
        """
        self.__con = None
        self.__aliases = {}

        # configparser related code
        config = configparser.ConfigParser()
        config.read(os.path.expanduser(os.path.expandvars(config_filename)))
        self.__rh_path = os.path.expanduser(os.path.expandvars(config.get(
            'PATHS',
            'rh_path',
            fallback=rh_path)))
        self.__rgbd_path = os.path.expanduser(os.path.expandvars(config.get(
            'PATHS',
            'rgbd_path',
            fallback=rgbd_path)))
        self.__scene_path = os.path.expanduser(os.path.expandvars(config.get(
            'PATHS',
            'scene_path',
            fallback=scene_path)))
        self.__wspc_path = os.path.expanduser(os.path.expandvars(config.get(
            'PATHS',
            'wspc_path',
            fallback=wspc_path)))
        self.__db_filename = os.path.expanduser(os.path.expandvars(config.get(
            'PATHS',
            'db_filename',
            fallback=db_filename)))

        logger.debug('rh_path     : {}', self.__rh_path)
        logger.debug('rgbd_path   : {}', self.__rgbd_path)
        logger.debug('scene_path  : {}', self.__scene_path)
        logger.debug('wspc_path   : {}', self.__wspc_path)
        logger.debug('db_filename : {}', self.__db_filename)

        # Initialization functions
        try:
            self.__open_dataset()
        except Exception as generic_error:
            logger.error("object cannot be instantiated")
            # return None
            raise generic_error

    def __del__(self):
        """Delete instance variables."""

    def __open_dataset(self):
        """Make the connection with the database."""
        db_full_path = os.path.join(self.__rh_path, self.__db_filename)
        logger.debug("db_full_path: {}", db_full_path)

        try:
            db_full_filename = 'file:'+db_full_path+'?mode=rw'
            logger.debug("db_full_filename: {}", db_full_filename)
            self.__con = sqlite3.connect(db_full_filename, uri=True)
            logger.success("Connection is established: {}", self.__db_filename)
        except sqlite3.Error as sqlite_error:
            logger.error("Error while trying to open database: {}",
                         sqlite_error.args[0])
            # sys.exit() , quit() , exit() ,raise SystemExit
            # os._exit(1)
            raise

    def __close_dataset(self):
        """Close the connection with the database."""
        self.__con.close()
        logger.info("The connection with the database has been successfully \
                     closed")

    def __get_temp_sql_object_names(self):
        """Return a list with temporary/internal created views."""
        return self.select_column('tbl_name', 'sqlite_temp_master')

    def get_con(self):
        """Return the sql connection variable."""
        return self.__con

    def get_path_vars(self):
        """Return path variables.

        Returns:
            array: [rh_path, rgbd_path, scene_path, wspc_path, db_filename]
        """
        return [self.__rh_path,
                self.__rgbd_path,
                self.__scene_path,
                self.__wspc_path,
                self.__db_filename]
    """
    Framework
    """
    def get_homes(self):
        """Return a dataframe with home names."""
        sql_str = 'select * from rh_homes;'
        return self.query(sql_str)

    def get_home_sessions(self):
        """Return a dataframe with home session names."""
        sql_str = 'select * from rh_home_sessions;'
        return self.query(sql_str)

    def get_rooms(self):
        """Return a dataframe with room names."""
        sql_str = 'select * from rh_rooms;'
        return self.query(sql_str)

    def get_room_types(self):
        """Return a dataframe with room type names."""
        sql_str = 'select * from rh_room_types;'
        return self.query(sql_str)

    def get_sensors(self):
        """Return a dataframe with sensor names."""
        sql_str = 'select * from rh_sensors;'
        return self.query(sql_str)

    def get_sensor_types(self):
        """Return a dataframe with sensor type names."""
        sql_str = 'select * from rh_sensor_types;'
        return self.query(sql_str)

    def get_hometopo(self):
        """Return a dataframe with home topo relationships."""
        sql_str = ('''
        select
            rh_homes.id as home_id,
            rh_homes.name as home_name,
            rh_rooms1.id as room1_id,
            rh_rooms1.name as room1_name,
            rh_rooms2.id as room2_id,
            rh_rooms2.name as room2_name
        from rh_hometopo
        inner join rh_homes on rh_hometopo.home_id = rh_homes.id
        inner join rh_rooms as rh_rooms1 on rh_rooms1.id = rh_hometopo.room1_id
        inner join rh_rooms as rh_rooms2 on rh_rooms2.id = rh_hometopo.room2_id
        '''
                   )
        return self.query(sql_str)

    def get_locators(self):
        """
        Return a dataframe with main indexes values (id and name).

        i.e., home_session, home, room and home_subsession
        """
        sql_str = """
        select
            home_session_id, rh_home_sessions.name as home_session_name,
            rh_raw.home_id, rh_homes.name as home_name,
            rh_raw.room_id, rh_rooms.name as room_name,
            rh_raw.home_subsession_id
        from rh_raw
        inner join rh_home_sessions on home_session_id = rh_home_sessions.id
        inner join rh_homes on rh_raw.home_id = rh_homes.id
        inner join rh_rooms on rh_raw.room_id = rh_rooms.id
        group by
            home_session_id,
            rh_raw.home_id,
            rh_raw.room_id,
            rh_raw.home_subsession_id

        order by
            rh_raw.home_session_id
        """

        df_rows = pd.read_sql_query(sql_str, self.__con)

        return df_rows

    """
    Captured data
    """
    def get_sensor_observations(self, arg='full'):
        """Return RGB-D and Laser observations as a pandas dataframe.

        The data provided by R@H2 have been collected within rooms of five
        dwellings (homes) Raw data were collected in different sessions and
        sub-sessions, each one containing a number of sequences of RGB-D
        observations and laser scans. These sequences were gathered by
        teleoperating the robot to fully inspect each individual room.

        Sensor data comprises ~75 minutes of recorded data collected in
        different sessions.

        These data include:

        RGB-D observations from the four RGB-D cameras, including intensity
        images, depth images, and 3D point clouds.

        Laser scanner data: 2D observations from the laser scanner captured in
        the inspected rooms.

        The R@H2 dataset contains those RGBD and Laser observations

        Args:
            arg:
                Can get five values:
                "full": indicates that the full set of observations will be
                    returned. This is the default value.
                "discarded": discarded observations are those that have not
                    been labeled (annotated).
                "lblrgbd": only labeled (annotated) RGB-D observations will
                    be returned.
                "lsrscan": laser scans will be returned.
                "rgbdlsr": both labeled RGB-D and laser scans will be returned.
        Returns:
            df: a Pandas dataframe with selected observations ordered by
                timestamp.
        """

        def get_full_data():
            """Return a dataframe containing the full set of observations.

            Returns:
                df: a Pandas dataframe with the data resulting from the
                    execution of the query according the SQL string.
            """
            sql_str = (
                '''
                select
                    id, time_stamp as timestamp,
                    home_session_id, home_subsession_id, home_id, room_id,
                    sensor_id, name as sensor_name,
                    sensor_pose_x, sensor_pose_y, sensor_pose_z,
                    sensor_pose_yaw, sensor_pose_pitch, sensor_pose_roll
                from rh2_sensor_observations
                order by time_stamp
                '''
            )
            logger.debug(sql_str)
            return self.query(sql_str)

        def get_discarded_data():
            """Return a dataframe containing the not processed observs.

            Returns:
                df: a Pandas dataframe with the data resulting from the
                    execution of the query according the SQL string.
            """
            sql_str = (
                '''
                select
                    id, time_stamp as timestamp,
                    home_session_id, home_subsession_id, home_id, room_id,
                    sensor_id, name as sensor_name,
                    sensor_pose_x, sensor_pose_y, sensor_pose_z,
                    sensor_pose_yaw, sensor_pose_pitch, sensor_pose_roll
                from rh2_sensor_observations
                where id < 100000
                order by time_stamp
                '''
            )
            logger.debug(sql_str)
            return self.query(sql_str)

        def get_lblrgbd_data():
            """Return a dataframe containing labeled RGB-D observations.

            Returns:
                df: a Pandas dataframe with the data resulting from the
                    execution of the query according the SQL string.
            """
            sql_str = (
                '''
                select
                    id, time_stamp as timestamp,
                    home_session_id, home_subsession_id, home_id, room_id,
                    sensor_id, name as sensor_name,
                    sensor_pose_x, sensor_pose_y, sensor_pose_z,
                    sensor_pose_yaw, sensor_pose_pitch, sensor_pose_roll
                from rh2_sensor_observations
                where id >= 100000 and id < 200000
                order by time_stamp
                '''
            )
            logger.debug(sql_str)
            return self.query(sql_str)

        def get_lsrscan_data():
            """Return a dataframe containing laser scans observations.

            Returns:
                df: a Pandas dataframe with the data resulting from the
                    execution of the query according the SQL string.
            """
            sql_str = (
                '''
                select
                    id, time_stamp as timestamp,
                    home_session_id, home_subsession_id, home_id, room_id,
                    sensor_id, name as sensor_name,
                    sensor_pose_x, sensor_pose_y, sensor_pose_z,
                    sensor_pose_yaw, sensor_pose_pitch, sensor_pose_roll
                from rh2_sensor_observations
                where id >= 200000
                order by time_stamp
                '''
            )
            logger.debug(sql_str)
            return self.query(sql_str)

        def get_rgbd_lsr_data():
            """Return a dataframe containing labeled RGB-D and laser scans.

            Returns:
                df: a Pandas dataframe with the data resulting from the
                    execution of the query according the SQL string.
            """
            sql_str = (
                '''
                select
                    id, time_stamp as timestamp,
                    home_session_id, home_subsession_id, home_id, room_id,
                    sensor_id, name as sensor_name,
                    sensor_pose_x, sensor_pose_y, sensor_pose_z,
                    sensor_pose_yaw, sensor_pose_pitch, sensor_pose_roll
                from rh2_sensor_observations
                where id >= 100000 and id < 300000
                order by time_stamp
                '''
            )
            logger.debug(sql_str)
            return self.query(sql_str)

        switcher = {
            "full":      get_full_data,
            "discarded": get_discarded_data,
            "lblrgbd":   get_lblrgbd_data,
            "lsrscan":   get_lsrscan_data,
            "rgbdlsr":   get_rgbd_lsr_data,
        }
        func = switcher.get(arg, lambda: "Invalid argument")

        df = func()

        return df

    def id2name(self, id, arg='r'):
        """Return the name corresponding to an id.

        The R@H2 database contains observations gathered from sensors of a
        mobile robot. Furthermore, it contains data about the framework where
        the data were gathered. That framework data is composed of some tables:

        - rh_homes
        - rh_home_sessions
        - rh_rooms
        - rh_sensors
        - rh_room_types
        - rh_sensor_types
        - rh_objects
        - rh_object_types

        All those tables have registries that associates ids (integer numbers)
        with names (descriptor strings).

        This functions takes an argument depicting the framework table to be
        searched and returns the name for the provided id.

        Args:
            id: the id number to be searched ('r' is the default value)
            arg: a string indicating the framework table to be searched:
            "h"            : rh_homes
            "hs"           : rh_home_sessions
            "r"            : rh_rooms
            "rt"           : rh_room_types
            "s"            : rh_sensors
            "st"           : rh_sensor_types
            "o"            : rh_objects
            "ot"           : rh_object_types
            "home"         : rh_homes
            "home_session" : rh_home_sessions
            "room"         : rh_rooms
            "room_type"    : rh_room_types
            "sensor"       : rh_sensors
            "sensor_type"  : rh_sensor_types
            "object"       : rh_objects
            "object_type"  : rh_object_types

        Returns:
            name: a string name associated with the id.
        """

        def get_home_name():
            """Return a dataframe containing the name corresponding to the id.

            Returns:
                df: a Pandas dataframe with the data resulting from the
                    execution of the query according the SQL string.
            """
            sql_str = (
                f'''
                select name
                from rh_homes
                where id = {id}
                '''
            )
            logger.debug(sql_str)
            return self.query(sql_str)

        def get_home_session_name():
            """Return a dataframe containing the name corresponding to the id.

            Returns:
                df: a Pandas dataframe with the data resulting from the
                    execution of the query according the SQL string.
            """
            sql_str = (
                f'''
                select name
                from rh_home_sessions
                where id = {id}
                '''
            )
            logger.debug(sql_str)
            return self.query(sql_str)

        def get_room_name():
            """Return a dataframe containing the name corresponding to the id.

            Returns:
                df: a Pandas dataframe with the data resulting from the
                    execution of the query according the SQL string.
            """
            sql_str = (
                f'''
                select name
                from rh_rooms
                where id = {id}
                '''
            )
            logger.debug(sql_str)
            return self.query(sql_str)

        def get_sensor_name():
            """Return a dataframe containing the name corresponding to the id.

            Returns:
                df: a Pandas dataframe with the data resulting from the
                    execution of the query according the SQL string.
            """
            sql_str = (
                f'''
                select name
                from rh_sensors
                where id = {id}
                '''
            )
            logger.debug(sql_str)
            return self.query(sql_str)

        def get_room_type_name():
            """Return a dataframe containing the name corresponding to the id.

            Returns:
                df: a Pandas dataframe with the data resulting from the
                    execution of the query according the SQL string.
            """
            sql_str = (
                f'''
                select name
                from rh_room_types
                where id = {id}
                '''
            )
            logger.debug(sql_str)
            return self.query(sql_str)

        def get_sensor_type_name():
            """Return a dataframe containing the name corresponding to the id.

            Returns:
                df: a Pandas dataframe with the data resulting from the
                    execution of the query according the SQL string.
            """
            sql_str = (
                f'''
                select name
                from rh_sensor_types
                where id = {id}
                '''
            )
            logger.debug(sql_str)
            return self.query(sql_str)

        def get_object_name():
            """Return a dataframe containing the name corresponding to the id.

            Returns:
                df: a Pandas dataframe with the data resulting from the
                    execution of the query according the SQL string.
            """
            sql_str = (
                f'''
                select name
                from rh_objects
                where id = {id}
                '''
            )
            logger.debug(sql_str)
            return self.query(sql_str)

        def get_object_type_name():
            """Return a dataframe containing the name corresponding to the id.

            Returns:
                df: a Pandas dataframe with the data resulting from the
                    execution of the query according the SQL string.
            """
            sql_str = (
                f'''
                select name
                from rh_object_types
                where id = {id}
                '''
            )
            logger.debug(sql_str)
            return self.query(sql_str)

        switcher = {
            "h":            get_home_name,
            "hs":           get_home_session_name,
            "r":            get_room_name,
            "rt":           get_room_type_name,
            "s":            get_sensor_name,
            "st":           get_sensor_type_name,
            "o":            get_object_name,
            "ot":           get_object_type_name,
            "home":         get_home_name,
            "home_session": get_home_session_name,
            "room":         get_room_name,
            "room_type":    get_room_type_name,
            "sensor":       get_sensor_name,
            "sensor_type":  get_sensor_type_name,
            "object":       get_object_name,
            "object_type":  get_object_type_name,
        }
        func = switcher.get(arg, lambda: "Invalid argument")

        return func().iat[0, 0]

    def name2id(self, name, arg='r'):
        """Return the id corresponding to a name.

        The R@H2 database contains observations gathered from sensors of a
        mobile robot. Furthermore, it contains data about the framework where
        the data were gathered. That framework data is composed of some tables:

        - rh_homes
        - rh_home_sessions
        - rh_rooms
        - rh_sensors
        - rh_room_types
        - rh_sensor_types
        - rh_objects
        - rh_object_types

        All those tables have registries that associates ids (integer numbers)
        with names (descriptor strings).

        This functions takes an argument depicting the framework table to be
        searched and returns the name for the provided id.

        Args:
            id: the id number to be searched ('r' is the default value)
            arg: a string indicating the framework table to be searched:
            "h"            : rh_homes
            "hs"           : rh_home_sessions
            "r"            : rh_rooms
            "rt"           : rh_room_types
            "s"            : rh_sensors
            "st"           : rh_sensor_types
            "o"            : rh_objects
            "ot"           : rh_object_types
            "home"         : rh_homes
            "home_session" : rh_home_sessions
            "room"         : rh_rooms
            "room_type"    : rh_room_types
            "sensor"       : rh_sensors
            "sensor_type"  : rh_sensor_types
            "object"       : rh_objects
            "object_type"  : rh_object_types

        Returns:
            id: an identification number associated with the name.
        """

        def get_home_id():
            """Return a dataframe containing the id corresponding to the name.

            Returns:
                df: a Pandas dataframe with the data resulting from the
                    execution of the query according the SQL string.
            """
            sql_str = (
                f'''
                select id
                from rh_homes
                where name like '%{name}%'
                '''
            )
            logger.debug(sql_str)
            return self.query(sql_str)

        def get_home_session_id():
            """Return a dataframe containing the id corresponding to the name.

            Returns:
                df: a Pandas dataframe with the data resulting from the
                    execution of the query according the SQL string.
            """
            sql_str = (
                f'''
                select id
                from rh_home_sessions
                where name like '%{name}%'
                '''
            )
            logger.debug(sql_str)
            return self.query(sql_str)

        def get_room_id():
            """Return a dataframe containing the id corresponding to the name.

            Returns:
                df: a Pandas dataframe with the data resulting from the
                    execution of the query according the SQL string.
            """
            sql_str = (
                f'''
                select id
                from rh_rooms
                where name like '%{name}%'
                '''
            )
            logger.debug(sql_str)
            return self.query(sql_str)

        def get_sensor_id():
            """Return a dataframe containing the id corresponding to the name.

            Returns:
                df: a Pandas dataframe with the data resulting from the
                    execution of the query according the SQL string.
            """
            sql_str = (
                f'''
                select id
                from rh_sensors
                where name like '%{name}%'
                '''
            )
            logger.debug(sql_str)
            return self.query(sql_str)

        def get_room_type_id():
            """Return a dataframe containing the id corresponding to the name.

            Returns:
                df: a Pandas dataframe with the data resulting from the
                    execution of the query according the SQL string.
            """
            sql_str = (
                f'''
                select id
                from rh_room_types
                where name like '%{name}%'
                '''
            )
            logger.debug(sql_str)
            return self.query(sql_str)

        def get_sensor_type_id():
            """Return a dataframe containing the id corresponding to the name.

            Returns:
                df: a Pandas dataframe with the data resulting from the
                    execution of the query according the SQL string.
            """
            sql_str = (
                f'''
                select id
                from rh_sensor_types
                where name like '%{name}%'
                '''
            )
            logger.debug(sql_str)
            return self.query(sql_str)

        def get_object_id():
            """Return a dataframe containing the id corresponding to the name.

            Returns:
                df: a Pandas dataframe with the data resulting from the
                    execution of the query according the SQL string.
            """
            sql_str = (
                f'''
                select id
                from rh_objects
                where name like '%{name}%'
                '''
            )
            logger.debug(sql_str)
            return self.query(sql_str)

        def get_object_type_id():
            """Return a dataframe containing the id corresponding to the name.

            Returns:
                df: a Pandas dataframe with the data resulting from the
                    execution of the query according the SQL string.
            """
            sql_str = (
                f'''
                select id
                from rh_object_types
                where name like '%{name}%'
                '''
            )
            logger.debug(sql_str)
            return self.query(sql_str)

        switcher = {
            "h": get_home_id,
            "hs": get_home_session_id,
            "r": get_room_id,
            "rt": get_room_type_id,
            "s": get_sensor_id,
            "st": get_sensor_type_id,
            "o": get_object_id,
            "ot": get_object_type_id,
            "home": get_home_id,
            "home_session": get_home_session_id,
            "room": get_room_id,
            "room_type": get_room_type_id,
            "sensor": get_sensor_id,
            "sensor_type": get_sensor_type_id,
            "object": get_object_id,
            "object_type": get_object_type_id,

        }
        func = switcher.get(arg, lambda: "Invalid argument")

        return func().iat[0, 0]

    """
    RGBD data
    """
    def get_RGBD_files(self, id):
        """
        Return the full path of RGB and D files for an observation.

        Args:
            id: the observation identification number to search for.

        Returns:
            An array with two strings:
                rgb_file: a string containing the full path of the RGB image
                    file corresponding to the observation.
                depth_file: a string containing the full path of the D(epth)
                    image file corresponding to the observation.
        """
        sql_str = (
            f'''
            select
              new_path as local_path,
              new_file_2 as rgb_file,
              new_file_1 as depth_file
            from rh2_old2new_rgbd_files
            where id = {id}
            '''
        )
        logger.debug(sql_str)
        # self.query(sql_str).values.flatten().tolist()
        # self.query(sql_str).loc[0,:].tolist()
        logger.debug('rh_rgbd_path: {}', self.__rgbd_path)
        files_path = self.query(sql_str).loc[0, :].tolist()
        depth_file = os.path.join(self.__rgbd_path,
                                  files_path[0],
                                  files_path[2])
        rgb_file = os.path.join(self.__rgbd_path,
                                files_path[0],
                                files_path[1])
        return [rgb_file, depth_file]

    def __get_RGBD_labels(self, id):
        """
        Return a dataframe with labels for the observation referenced by id.

        SQL query

        select * from rh_lblrgbd_labels
            where sensor_observation_id = id

        Parameters
        ----------
        id : int
            The primary key value to identify a row in the table
            rh_lbl_rgbd_labels.

        Returns
        -------
        A dataframe with the query result. An empty dataframe is returned when
        no rows are available, i.e., when the sensor observation does not
        belong to rh_lblrgbd (labelled rgbd)

        Note
        ----
        This is a local function
        """
        sql_str = (
            f'''
            select *
            from rh_lblrgbd_labels
             where sensor_observation_id = {id}
            '''
        )
        logger.debug(sql_str)
        return self.query(sql_str)

    def __get_Labels_file(self, id):
        """Return the <id>_labels.txt full path filename."""
        sql_str = (
            f'''
            select
              new_path as local_path,
              new_file_3 as labels_file
            from rh2_old2new_rgbd_files
            where id = {id}
            '''
        )
        logger.debug(sql_str)
        files_path = self.query(sql_str).loc[0, :].tolist()
        labels_file = os.path.join(self.__rgbd_path,
                                   files_path[0],
                                   files_path[1])
        return labels_file

    def __get_label_mask_array(self, l_f):
        """Return a numpy array from the <id>_labels.txt."""
        mask = []
        with open(l_f, "r") as file_handler:
            line = file_handler.readline()
            while line:
                words = line.strip().split()
                if words[0][0] != '#':
                    num_of_labels = int(words[0])
                    break
                line = file_handler.readline()

            for _ in range(num_of_labels):
                line = file_handler.readline()
                words = line.strip().split()

            num_of_rows = 0
            line = file_handler.readline()
            while line:
                num_of_rows += 1
                words = line.strip().split()
                mask.append(list(map(int, words)))
                line = file_handler.readline()

        # logger.debug("mask height: {}", len(mask))
        # logger.debug("mask width : {}", len(mask[0]))

        mask_array = np.array(mask)
        mask_array = np.rot90(mask_array)

        logger.debug("\nmask array size: {} (# rows, # cols)", mask_array.shape)
        logger.debug("\nmask array max value: {}", np.amax(mask_array))

        return mask_array

    def __decompose_label_mask_array(self, label_mask_array, labels):
        """
        Return a list of binary 2D numpy.ndarray arrays (pixels being 1 and 0).

        A 2D numpy.ndarray array per label

        Input
        =====
        label_mask_array: a numpy.ndarray of numpy.int64 values
                    Each value of the matrix must represents a pixel by means
                    of an int64. This number must be interpreted as a binary
                    number where each bit with value 1 means that the element
                    belongs to the label which label_id matchs the bit position.
                    For example, if an element has the value 1010, this means
                    that it belongs to the labels with id 1 and 3.
                    In this way an element can belong to more than one label. 

        labels: a pandas series with the 'local_id' column of a
                rh_lblrgbd_labels dataframe for a rh_lblrgbd record. This
                series always is a sequence of number in the range 1 to number
                of labels, i.e. 1,2,...,# labels

        Output
        ======
        masks: a 1-column dataframe of 2D binary arrays. Each item is a 2D binary
               numpy.nparray (pixels being 1s and 0s) corresponding to a label,
               where each matrix value <numpy.int8> corresponds to a pixel,
               being 1 if belongs to the label and 0 otherwise.
        """
        masks = []
        for label in labels:
            arr = label_mask_array & (2**(label))
            np.clip(arr, 0, 1, out=arr)
            arr = np.uint8(arr[:, 2:-2])
            masks.append(arr)
        logger.debug("\nmask list with {} items of {} binary arrays", len(masks), masks[0].shape)

        # Transform masks list to a one-column pandas dataframe
        mask_df = pd.DataFrame(columns=['mask'])
        mask_df['mask'] = mask_df['mask'].astype(object)
        mask_df.loc[:, 'mask'] = masks
        return mask_df

    def get_RGBD_labels(self, id, masks=True):
        """
        Return a list of one binary array per label of the RGBD image.

        The image is referenced by its id
        """
        # Get a dataframe from rh_lblrgbd_labels with labels of the id
        labels = self.__get_RGBD_labels(id)

        # By default 2D binary masks column is added to the labels dataframe
        if masks:
            # Get the <id>_labels.txt full path filename
            l_f = self.__get_Labels_file(id)
            # Get the label_mask_array from the <id>_labels.txt file
            label_mask_array = self.__get_label_mask_array(l_f)
            # logger.debug("\nlabels['local_id']: \n{}", labels['local_id'])
            # Get a 1-column dataframe of one binary array per label
            label_masks = self.__decompose_label_mask_array(label_mask_array, labels['local_id'])
            # Return the labels dataframe concatenated with the new label_masks
            # 1-column dataframe
            labels = pd.concat([labels, label_masks], axis=1)

        return labels

    def get_RGBD_sensor_size(self):
        """Return RGB sensor pixel dimension."""
        RGBD_sensor_size = {
            'w': 240,
            'h': 320
        }
        return RGBD_sensor_size


    def get_RGBD_fps(self):
        """Return RGBD cameras frame rate.
        
        The returned value is a fixed quantity resulting from
        previous average computations.
        A more precise value can be calculated by dividing the
        time between the first frame and the last frame by the
        number of frames.
        """

        return 10.72


    """
    Laser Scanner data
    """
    def get_laser_scan(self, id):
        """
        Return a dataframe with a laser scan for the observation.

        The observation is referenced by id

        SQL query

        select * from rh_lsrscan_scans
            where sensor_observation_id = id

        Parameters
        ----------
        id : int
            The primary key value to identify a row in the table
            rh_lbl_rgbd_labels.

        Returns
        -------
        A dataframe with the query result. An empty dataframe is returned when
        no rows are available, i.e., when the sensor observation does not
        belong to rh_lblrgbd (labelled rgbd)
        """
        # if id < 200000:
        #     scan_table_name = 'rh_scans'
        # else:
        #     scan_table_name = 'rh_lsrscan_scans'

        scan_table = lambda id: 'rh_raw_scans' if id < 200000 else 'rh_lsrscan_scans'

        sql_str = (
            f'''
            select *
            from {scan_table(id)}
             where sensor_observation_id = {id}
            '''
        )
        logger.debug(sql_str)

        df = self.query(sql_str)

        # Laser scanner aperture and max distance are hard coded as it's in the
        # original dataset. Nevertheless are included as columns in the
        # rh2_sensor_observations
        df.aperture = 4.1847
        df.max_range = 5.6
        df.no_of_shots = 682

        return df

    """
    Scenes
    """
    def get_scenes(self):
        """Return a dataframe with scenes."""
        sql_str = (
            f'''
            select
              rh_lblscene.id,
              home_session_id, home_subsession_id, home_id, room_id,
              '{self.__scene_path}' || '/' || new_path || '/' || new_file as scene_file
            from rh_lblscene
            inner join rh2_old2new_scene_files on rh_lblscene.id = rh2_old2new_scene_files.id
            '''
        )
        logger.debug(sql_str)
        scenes = self.query(sql_str)
        return scenes

    def get_scene_labels(self, id, obj=False):
        """
        Return a dataframe with a scene labels for the observation.

        The observation is referenced by id

        SQL query

        select * from rh_lsrscan_scans
            where sensor_observation_id = id

        Parameters
        ----------
        id : int
            The primary key value to identify a row in the table
            rh_lbl_rgbd_labels.

        Returns
        -------
        A dataframe with the query result. An empty dataframe is returned when
        no rows are available, i.e., when the sensor observation does not
        belong to rh_lblrgbd (labelled rgbd)
        """
        # if id < 200000:
        #     scan_table_name = 'rh_scans'
        # else:
        #     scan_table_name = 'rh_lsrscan_scans'

        labels_table = lambda obj: 'rh2_scene_bb_objects' if obj else 'rh_lblscene_bboxes'

        if obj:
            sql_str = (
                f'''
                select
                    id,
                    local_id,
                    scene_id,
                    object_id,
                    object_name,
                    object_type_id,
                    bb_pose_x,
                    bb_pose_y,
                    bb_pose_z,
                    bb_pose_yaw,
                    bb_pose_pitch,
                    bb_pose_roll,
                    bb_corner1_x,
                    bb_corner1_y,
                    bb_corner1_z,
                    bb_corner2_x,
                    bb_corner2_y,
                    bb_corner2_z,
                    planarity,
                    scatter,
                    linearity,
                    min_height,
                    max_height,
                    centroid_x,
                    centroid_y,
                    centroid_z,
                    volume,
                    biggest_area,
                    orientation,
                    hue_mean,
                    saturation_mean,
                    value_mean,
                    hue_stdv,
                    saturation_stdv,
                    value_stdv,
                    hue_histogram_0,
                    hue_histogram_1,
                    hue_histogram_2,
                    hue_histogram_3,
                    hue_histogram_4,
                    value_histogram_0,
                    value_histogram_1,
                    value_histogram_2,
                    value_histogram_3,
                    value_histogram_4,
                    saturation_histogram_0,
                    saturation_histogram_1,
                    saturation_histogram_2,
                    saturation_histogram_3,
                    saturation_histogram_4
                from {labels_table(obj)}
                where scene_id = {id}
                '''
            )
        else:
            sql_str = (
                f'''
                select *
                from {labels_table(obj)}
                where scene_id = {id}
                '''
            )

        logger.debug(sql_str)
        df = self.query(sql_str)
        return df

    """
    Observations
    """
    def get_observations(self):
        """Return a dataframe with observations.

        For rooms there is a set of RGBD observations that has been
        characterized considering averaged features from past observations, as
        well as features of the closest laser scan in time. The number of room
        features is 48 and the number of scan features is 9.

        Unfortunately, this set of observations should be considered
        experimental in the sense that the timestamp data has been lost,
        preventing the identification of the sensor observations from which
        they where extracted. Nevertheless, RGBD observations are sequentially
        ordered.
        """
        sql_str = (
            '''
            select *
            from rh_observations
            '''
        )
        logger.debug(sql_str)
        observations = self.query(sql_str)
        return observations

    def get_objects(self):
        """Return a dataframe with all objects.

        The objects extracted from the observations have also been
        characterized. For each object the number of features is 32.
        """
        sql_str = (
            '''
            select *
            from rh_objects
            '''
        )
        logger.debug(sql_str)
        objects = self.query(sql_str)
        return objects

    def get_object_relations(self, id=None):
        """Return a dataframe with object relationships.

        The relation between objects has also been calculated and characterized
        with 11 features.

        The returned dataframe contains the relationships an object is implicit
        in.

        Args:
            id: integer number with the identification of an object. If this
                value is empty this functions returns a dataframe with all
                relations in the database.

        """
        sql_str = (
            '''
            select *
            from rh_relations
            '''
        )

        where_str = (
            f'''
            where obj1_id == {id} or obj2_id == {id}
            '''
        )

        if (id is not None):
            sql_str = sql_str + where_str

        logger.debug(sql_str)
        object_relations = self.query(sql_str)
        return object_relations

    def get_objects_in_observation(self, id):
        """Return a dataframe with objects in observation.

        The returned objects are contained in a characterized RGBD observation.
        """
        sql_str = (
            f'''
            select rh_objects.*
            from rh_objects_in_observation
            inner join rh_objects on rh_objects_in_observation.object_id == rh_objects.id
            where  rh_objects_in_observation.observation_id == {id}
            '''
        )
        logger.debug(sql_str)
        objects = self.query(sql_str)
        return objects

    """
    Stuff
    """
    def select_column(self, column_name, table_name):
        """
        Return a dataframe with grouped column values.

        (without repetition)
        """
        # Get a cursor to execute SQLite statements
        sql_str = (f"select {column_name}  from {table_name} group by {column_name};")
        df_rows = pd.read_sql_query(sql_str, self.__con)
        return df_rows

    def query(self, sql, df=True):
        """
        Execute a sqlquery over robotathome database.

        Parameters
        ----------
        sql: can be a string with a sql query or a file name that contains the
             sql query.
        df:  boolean indicating if result is returned as a DataFrame (True) or
             as a sqlite row list (False).  This option (False) is mandatory if
             the query string has more than one sql command, i.e., it's a
             script.

        Returns
        -------
        ans: a DataFrame or a sqlite row list.
        """
        if os.path.isfile(sql):
            script = open(sql, 'r')
            query = script.read()
        else:
            query = sql

        if df:
            ans = pd.read_sql_query(query, self.__con)
        else:
            cur = self.__con.cursor()
            cur.executescript(query)
            ans = cur.fetchall()

        if os.path.isfile(sql):
            script.close()

        return ans

    """
    Lab
    """



#!/usr/bin/env python
# -*- coding: utf-8; buffer-read-only: t -*-

__author__ = "Gregorio Ambrosio"
__contact__ = "gambrosio[at]uma.es"
__copyright__ = "Copyright 2021, Gregorio Ambrosio"
__date__ = "2021/10/25"
__license__ = "MIT"

__all__ = ['say_hello']

def say_hello():
    print("Hello World")




#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Setup for RobotAtHome2
"""

import os
import sys

from setuptools import find_packages, setup

def read(rel_path):
    """ Docstring """
    here = os.path.abspath(os.path.dirname(__file__))
    with open(os.path.join(here, rel_path), 'r') as fp:
        return fp.read()

def get_version(rel_path):
    """ Docstring """
    for line in read(rel_path).splitlines():
        if line.startswith('__version__'):
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]
    raise RuntimeError("Unable to find version string.")

try:
    with open("README.md", "r") as fh:
        LONG_DESCRIPTION = fh.read()
except Exception:
    LONG_DESCRIPTION = ''

# Platform specific code
if sys.platform.startswith('win'):
    PYTHON_REQUIRES = '<3.7'
elif sys.platform.startswith('linux'):
    PYTHON_REQUIRES = '>=3.7'

setup(
    name="robotathome",
    version=get_version("robotathome/__init__.py"),
    description="This package provides a Python Toolbox with a set of functions to assist in the management of Robot@Home2 Dataset",
    long_description = LONG_DESCRIPTION,
    long_description_content_type="text/markdown",

    license='MIT',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    url="https://github.com/goyoambrosio/RobotAtHome2",
    keywords=('toolbox '
              'dataset '
              'database '
              'relational model '
              'mobile robotics '
              'computer vision '
              ),

    author="G. Ambrosio-Cestero",
    author_email="gambrosio@uma.es",
    platforms=['Windows', 'Linux'],

    packages=find_packages(),

    install_requires=[
        "humanize",
        "click",
        "urllib3",
        "requests",
        "tqdm",
        "loguru",
        "pandas",
        "numpy"
    ],
    extras_require={
        'full':['matplotlib>=3', 'notebook','ipywidgets','opencv-python','opencv-contrib-python'],
        'interactive': ['matplotlib>=3', 'notebook', 'ipywidgets'],
        'cv': ['opencv-python','opencv-contrib-python']
    },
    python_requires=PYTHON_REQUIRES,

    tests_require=['unittest'],
    # These files will be located at ~/<anaconda-dir>/envs/<env-name>/[robotathome/schema|robotathome/notebooks|...]
    data_files=[
        ('robotathome/schema',
         ['rh_schema_diagram/rh_schema_brief.pdf',
          'rh_schema_diagram/rh_schema_full.pdf']
         ),
        ('robotathome/notebooks',
         ['notebooks/05-Google-colab-drive.ipynb',
          'notebooks/10-Download-and-install.ipynb',
          'notebooks/20-Before-starting-the-logging-system.ipynb',
          'notebooks/30-Getting-started-Framework-data.ipynb',
          'notebooks/40-Captured-data.ipynb',
          'notebooks/50-RGBD-observations.ipynb',
          'notebooks/60-Lsrscan-observations.ipynb',
          'notebooks/70-Scenes.ipynb',
          'notebooks/80-Characterized-observations.ipynb',
          'notebooks/90-The-config-file.ipynb',
          'notebooks/100-Iterating-over-RGBD-images.ipynb',
          'notebooks/110-Concatenating-images-from-multiple-cameras.ipynb',
          'notebooks/120-Making-a-video.ipynb',
          'notebooks/130-Processing-images-with-YOLO.ipynb',
          'notebooks/140-Loading-dataset-in-Google-Colab.ipynb']
         )
    ],
    # sql files from robotathome package will be included in the installation package
    # located at ~/<anaconda-dir>/envs/<env-name>/lib/<python>/site-packages/robotathome
    package_data={'robotathome': ['*.sql']},

)



#!/usr/bin/env python
# -*- coding: utf-8; buffer-read-only: t -*-

"""
Get version for RobotAtHome2 package
"""

import os

__all__=['get_version_str']

def _read(rel_path):
    """ Docstring """
    here = os.path.abspath(os.path.dirname(__file__))
    with open(os.path.join(here, rel_path), 'r') as fp:
        return fp.read()

def _get_version(rel_path):
    """ Docstring """
    for line in _read(rel_path).splitlines():
        if line.startswith('__version__'):
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]
    raise RuntimeError("Unable to find version string.")

def get_version_str():
    """ Docstring """

    s = "Robot@Home2 Dataset Toolbox(v" + _get_version("__init__.py") + ")"

    return s



#!/usr/bin/env python
# -*- coding: utf-8; buffer-read-only: t -*-

__author__ = "Gregorio Ambrosio"
__contact__ = "gambrosio[at]uma.es"
__copyright__ = "Copyright 2021, Gregorio Ambrosio"
__date__ = "2021/07/27"
__license__ = "MIT"


import robotathome as rh
import mxnet as mx
from mxnet import image
from mxnet import context
import gluoncv as gcv
from gluoncv import model_zoo, data, utils


def lblrgbd_rgb_image_object_detection(self, so_id,
                                       model='yolo3_darknet53_coco'):
    bgr_img = self.get_rgb_image_from_lblrgbd(so_id)
    chw_img, class_names, nn_out = rh.object_detection_with_gluoncv(
        cv2.cvtColor(bgr_img, cv2.COLOR_BGR2RGB),
        model
    )
    return bgr_img, chw_img, class_names, nn_out

def lblrgbd_object_detection(self,
                             source='lblrgbd',
                             home_session_name='alma-s1',
                             home_subsession=0,
                             room_name='alma_masterroom1',
                             sensor_name='RGBD_1',
                             video_file_name=None,
                             model='yolo3_darknet53_coco',
                             gpu=False
                             ):
    """
    This functions ...
    """
    rows = self.get_sensor_observation_files(source,
                                             home_session_name,
                                             home_subsession,
                                             room_name,
                                             sensor_name)

    # Computing frames per second
    num_of_frames = len(rows)
    seconds = (rows.iloc[-1]['t'] - rows.iloc[0]['t']) / 10**7
    frames_per_second = num_of_frames / seconds
    rh.logger.debug("frames per second: {:.2f}", frames_per_second)

    # Get frame size
    img = self.get_rgb_image_from_lblrgbd(rows.iloc[0]['id'])
    img_h, img_w, _ = img.shape

    # Opening video file
    if video_file_name is None:
        video_file_name = ''.join(
            [
                home_session_name,
                '_', str(home_subsession),
                '_', room_name,
                '_', sensor_name,
                '_by_', model,
                dt.datetime.now().strftime("_%Y%m%d%H%M%S"),
                '.avi'
            ]
        )
    fourcc = cv2.VideoWriter_fourcc(*'MJPG')
    video_path_file_name = os.path.abspath(os.path.join(self.__wspc_path,
                                                        video_file_name)
                                           )
    rh.rename_if_exist(video_path_file_name)

    out = cv2.VideoWriter(video_path_file_name,
                          fourcc,
                          frames_per_second,
                          (img_w, img_h))

    ##############################################
    #                     NN
    ##############################################

    yolo_models = rh.get_yolo_models()
    rcnn_models = rh.get_rcnn_models()

    if model not in yolo_models + rcnn_models:
        raise Exception(f"Sorry, the model '{model}' is not allowed")

    #  Set context to cpu or gpu
    ctx_ = mx.context.gpu() if gpu else mx.context.cpu()

    # Load Pretrained Model from the CV model zoo
    net = gcv.model_zoo.get_model(model,
                                  pretrained=True,
                                  ctx=ctx_)
    class_names_ = net.classes
    nn_out_list = []
    i = 0
    for _, row in rows.iterrows():
        img = self.get_rgb_image_from_lblrgbd(row['id'])

        i += 1
        if rh.is_being_logged('INFO'):
            sys.stdout.write("\rProcessing frame %i of %i" % (i, len(rows)))
            sys.stdout.flush()

        short_edge_size = min(img.shape[0:2])
        if model in yolo_models:
            trnf_img, _ = gcv.data.transforms.presets.yolo.transform_test(mx.nd.array(img),
                                                                          short=short_edge_size)
        if model in rcnn_models:
            trnf_img, _ = gcv.data.transforms.presets.rcnn.transform_test(mx.nd.array(img),
                                                                          short=short_edge_size)
        class_ids, scores, bounding_boxs = net(trnf_img)

        df_nn_out = rh.nn_out2df(class_ids, scores, bounding_boxs)
        nn_out_list.append(df_nn_out)

        utils.viz.cv_plot_bbox(img,
                               bounding_boxs[0],
                               scores[0],
                               class_ids[0],
                               class_names=class_names_,
                               thresh=0.2,
                               linewidth=1)
        if rh.is_being_logged():
            cv2.imshow('Debug mode (press q to exit)', img)
            if cv2.waitKey(1) & 0xFF == ord('q'):
                break

        out.write(img)

    out.release()

    if rh.is_being_logged():
        cv2.destroyAllWindows()

    df_nn_out = pd.DataFrame(nn_out_list, columns=['class_ids',
                                                   'scores',
                                                   'bounding_boxs'])
    return df_nn_out, video_file_name

def process_with_yolo(self,
                      source='lblrgbd',
                      home_session_name='alma-s1',
                      home_subsession=0,
                      room_name='alma_masterroom1',
                      sensor_name='RGBD_1',
                      video_file_name=None,
                      gpu=False
                      ):

    """
    This functions ...
    """

    rh.logger.warning("This function is DEPRECATED, use lblrgbd_object_detection()")
    rh.logger.warning("with model='yolo3_darknet53_coco' instead ")

    rows = self.get_sensor_observation_files(source,
                                             home_session_name,
                                             home_subsession,
                                             room_name,
                                             sensor_name)

    # Computing frames per second
    num_of_frames = len(rows)
    seconds = (rows.iloc[-1]['t'] - rows.iloc[0]['t']) / 10**7
    frames_per_second = num_of_frames / seconds
    rh.logger.debug("frames per second: {:.2f}", frames_per_second)

    # Get frame size
    image_path = rows.iloc[0]['pth']
    file_name = rows.iloc[0]['f2']
    image_path_file_name = os.path.join(self.__rh_path,
                                        self.__rgbd_path,
                                        image_path,
                                        file_name)
    img = cv2.imread(image_path_file_name, cv2.IMREAD_COLOR)
    img_h, img_w, _ = img.shape

    # Opening video file
    if video_file_name is None:
        video_file_name = ''.join(
            [
                home_session_name,
                '_', str(home_subsession),
                '_', room_name,
                '_', sensor_name,
                '_by_YOLO',
                dt.datetime.now().strftime("_%Y%m%d%H%M%S"),
                '.avi'
            ]
        )
    fourcc = cv2.VideoWriter_fourcc(*'MJPG')
    video_path_file_name = os.path.abspath(os.path.join(self.__wspc_path,
                                                        video_file_name)
                                           )
    rh.rename_if_exist(video_path_file_name)

    out = cv2.VideoWriter(video_path_file_name,
                          fourcc,
                          frames_per_second,
                          (img_w, img_h))



    ##############################################
    #                     NN
    ##############################################


    #  get NN model
    ctx_ = context.gpu() if gpu else context.cpu()
    net = model_zoo.get_model('yolo3_darknet53_coco',
                              pretrained=True,
                              ctx=ctx_)

    nn_out = []
    i = 0
    for _, row in rows.iterrows():
        image_path = row['pth']
        file_name = row['f2']
        image_path_file_name = os.path.join(self.__rh_path,
                                            self.__rgbd_path,
                                            image_path,
                                            file_name)
        i += 1
        if rh.is_being_logged('INFO'):
            sys.stdout.write("\rProcessing frame %i of %i" % (i, len(rows)))
            sys.stdout.flush()

        # core
        try:
            img = image.imread(image_path_file_name)
        except:
            print('%s is not a valid raster image' % image_path_file_name)

        # long_edge_size = img.shape[0]
        short_edge_size = img.shape[1]

        x, img = data.transforms.presets.yolo.load_test(image_path_file_name,
                                                        short=short_edge_size)
        # rh.logger.debug('Shape of pre-processed image: {}', x.shape)
        class_ids, scores, bounding_boxs = net(x)

        # to DataFrame
        df_class_ids = pd.DataFrame(class_ids.asnumpy()[0].tolist(),
                                    columns=['class_ids'])
        df_scores = pd.DataFrame(scores.asnumpy()[0].tolist(),
                                 columns=['scores'])
        df_bounding_boxs = pd.DataFrame(
            bounding_boxs.asnumpy()[0].tolist(),
            columns=['xmin', 'ymin', 'xmax', 'ymax'])

        nn_out.append([df_class_ids,
                       df_scores,
                       df_bounding_boxs])

        utils.viz.cv_plot_bbox(img,
                               bounding_boxs[0],
                               scores[0],
                               class_ids[0],
                               class_names=net.classes,
                               thresh=0.2,
                               linewidth=1)
        if rh.is_being_logged():
            cv2.imshow('Debug mode (press q to exit)', img)
            if cv2.waitKey(1) & 0xFF == ord('q'):
                break

        out.write(img)

    out.release()

    if rh.is_being_logged():
        cv2.destroyAllWindows()

    df_nn_out = pd.DataFrame(nn_out, columns=['class_ids',
                                              'scores',
                                              'bounding_boxs'])
    return df_nn_out, video_file_name

def process_with_rcnn(self,
                      source='lblrgbd',
                      home_session_name='alma-s1',
                      home_subsession=0,
                      room_name='alma_masterroom1',
                      sensor_name='RGBD_1',
                      video_file_name=None,
                      gpu=False
                      ):

    """
    This functions ...
    """

    rh.logger.warning("This function is DEPRECATED, use lblrgbd_object_detection()")
    rh.logger.warning("with model='faster_rcnn_resnet50_v1b_coco' instead ")

    rows = self.get_sensor_observation_files(source,
                                             home_session_name,
                                             home_subsession,
                                             room_name,
                                             sensor_name)

    # Computing frames per second
    num_of_frames = len(rows)
    seconds = (rows.iloc[-1]['t'] - rows.iloc[0]['t']) / 10**7
    frames_per_second = num_of_frames / seconds
    rh.logger.debug("frames per second: {:.2f}", frames_per_second)

    # Get frame size
    image_path = rows.iloc[0]['pth']
    file_name = rows.iloc[0]['f2']
    image_path_file_name = os.path.join(self.__rh_path,
                                        self.__rgbd_path,
                                        image_path,
                                        file_name)
    img = cv2.imread(image_path_file_name, cv2.IMREAD_COLOR)
    img_h, img_w, _ = img.shape

    # Opening video file
    if video_file_name is None:
        video_file_name = ''.join(
            [
                home_session_name,
                '_', str(home_subsession),
                '_', room_name,
                '_', sensor_name,
                '_by_RCNN',
                dt.datetime.now().strftime("_%Y%m%d%H%M%S"),
                '.avi'
            ]
        )
    fourcc = cv2.VideoWriter_fourcc(*'MJPG')
    video_path_file_name = os.path.abspath(os.path.join(self.__wspc_path,
                                                        video_file_name)
                                           )
    out = cv2.VideoWriter(video_path_file_name,
                          fourcc,
                          frames_per_second,
                          (img_w, img_h))



    ##############################################
    #                     NN
    ##############################################


    #  get NN model
    # import mxnet as mx
    # ctx = mx.gpu() if gpu else mx.cpu()  # Set context
    # ctx = context.cpu()
    # ctx = context.cpu_pinned()
    # ctx = context.gpu(dev_id)
    ctx_ = context.gpu() if gpu else context.cpu()
    net = model_zoo.get_model('faster_rcnn_resnet50_v1b_coco',
                              pretrained=True,
                              ctx=ctx_) 
    nn_out = []
    i = 0
    for _, row in rows.iterrows():
        image_path = row['pth']
        file_name = row['f2']
        image_path_file_name = os.path.join(self.__rh_path,
                                            self.__rgbd_path,
                                            image_path,
                                            file_name)
        i += 1
        sys.stdout.write("\rProcessing frame %i of %i" % (i, len(rows)))
        sys.stdout.flush()

        # core
        try:
            img = image.imread(image_path_file_name)
        except:
            print('%s is not a valid raster image' % image_path_file_name)

        # long_edge_size = img.shape[0]
        short_edge_size = img.shape[1]

        x, img = data.transforms.presets.rcnn.load_test(image_path_file_name,
                                                        short=short_edge_size)
        # rh.logger.debug('Shape of pre-processed image: {}', x.shape)
        class_ids, scores, bounding_boxs = net(x)
        # to DataFrame
        df_class_ids = pd.DataFrame(class_ids.asnumpy()[0].tolist(),
                                    columns=['class_ids'])
        df_scores = pd.DataFrame(scores.asnumpy()[0].tolist(),
                                 columns=['scores'])
        df_bounding_boxs = pd.DataFrame(
            bounding_boxs.asnumpy()[0].tolist(),
            columns=['xmin', 'ymin', 'xmax', 'ymax'])

        nn_out.append([df_class_ids,
                       df_scores,
                       df_bounding_boxs])

        utils.viz.cv_plot_bbox(img,
                               bounding_boxs[0],
                               scores[0],
                               class_ids[0],
                               class_names=net.classes,
                               thresh=0.2,
                               linewidth=1)
        if rh.is_being_logged():
            cv2.imshow('Debug mode (press q to exit)', img)
            if cv2.waitKey(1) & 0xFF == ord('q'):
                break

        out.write(img)

    out.release()

    if rh.is_being_logged():
        cv2.destroyAllWindows()

    df_nn_out = pd.DataFrame(nn_out, columns=['class_ids',
                                              'scores',
                                              'bounding_boxs'])
    return df_nn_out, video_file_name


"""
Helpers
"""

def get_yolo_models():
    models = list(gcv.model_zoo.get_model_list())
    yolo_models = [models[i] for i in [176, 177, 179, 180, 182, 183]]
    return yolo_models

def get_rcnn_models():
    models = list(gcv.model_zoo.get_model_list())
    rcnn_models = [models[i] for i in [84, 86, 87, 88, 89, 91, 92, 93, 94, 95, 97]]
    return rcnn_models

def nn_out2df(class_ids, scores, bounding_boxs):
    # to DataFrame

    df_class_ids = pd.DataFrame(class_ids.asnumpy()[0].tolist(),
                                columns=['class_ids'])
    df_scores = pd.DataFrame(scores.asnumpy()[0].tolist(),
                             columns=['scores'])
    df_bounding_boxs = pd.DataFrame(
        bounding_boxs.asnumpy()[0].tolist(),
        columns=['xmin', 'ymin', 'xmax', 'ymax'])

    return [df_class_ids, df_scores, df_bounding_boxs]

def object_detection_with_gluoncv(img,
                                  model='yolo3_darknet53_coco',
                                  gpu=False):
    """ Using a pre-trained model for object detection using GluonCV

    Parameters
    ----------
    img: a preloaded image with HWC layout
    model: a string with the model to apply
    gpu: boolean indicating if gpu context must be used

    Returns
    -------
    nn_out: a list with three MXNet ndarrays [class_ids, scores, bounding_boxs]
    df_nn_out: a dataframe with the three MXNet ndarrays converted
               to dataframes
    """

    yolo_models = get_yolo_models()
    rcnn_models = get_rcnn_models()

    if model not in yolo_models + rcnn_models:
        raise Exception(f"Sorry, the model '{model}' is not allowed")

    #  Set context to cpu or gpu
    ctx_ = mx.context.gpu() if gpu else mx.context.cpu()

    # Load Pretrained Model from the CV model zoo
    net = gcv.model_zoo.get_model(model,
                                  pretrained=True,
                                  ctx=ctx_)

    # Transform the Image

    """Function that applies all of the necessary preprocessing steps for the
    yolo network

    Outputs:

    trnf_img    the transformed image that is ready to be given to the network.
    chw_img     a resized version of the image for plotting results
                - It’s in NCHW format instead of NHWC format
                - It’s an array of 32-bit floats instead of 8-bit integers
                - It’s normalized using the image net 1K statistics
                - It can be plotted because it’s in the CHW format that is
                  used by pipeline: plt.imshow(chw_image)

    Note about CHW/HCW layouts (https://github.com/Microsoft/CNTK/issues/276):

     It is essentially the same terminology that is used by cuDNN (see cuDNN API
     reference/manual). cuDNN is a very popular library which is supported by
     all popular DNN toolkits, not just CNTK, so we decided to use the same
     terminology to simplify migration between the toolkits. Note that cuDNN
     also uses N for a sample in a batch and D for the depth dimension used,
     for example, in 3D convolutions, so full description of the data layout
     may look like: NCHW

    H: height
    W: width
    C: channels (3 for color images, 1 for B&W images)

    CHW: RR...R, GG..G, BB..B
    HWC: RGB, RGB, ... RGB

    """
    short_edge_size = min(img.shape[0:2])

    if model in yolo_models:
        rh.logger.trace("yolo_model: {}", model)
        trnf_img, chw_img = gcv.data.transforms.presets.yolo.transform_test(mx.nd.array(img),
                                                                            short=short_edge_size)
    if model in rcnn_models:
        rh.logger.trace("rcnn_model: {}", model)
        trnf_img, chw_img = gcv.data.transforms.presets.rcnn.transform_test(mx.nd.array(img),
                                                                            short=short_edge_size)


    """ Make prediction
    It returns three MXNet ndarrays:
    1st array contains the object class indexes.
        shape : (1, 100, 1)
        1 image
        100 potential objects
        1 class index per object
    2nd array contains the object class probabilities.
        shape : (1, 100, 1)
        1 image
        100 potential objects
        1 object class probability (score)
    3rd array contains the object bounding box coordinates.
        shape : (1, 100, 4)
        1 image
        100 potential objects
        4 values for each object to define its bounding box:
            xmin, ymin, xmax, ymax

    net.classes is an array with coco classes
    -1 is a special value used to indicate there is no detected object
    """
    class_ids, scores, bounding_boxs = net(trnf_img)

    nn_out = [class_ids, scores, bounding_boxs]

    return chw_img, net.classes, nn_out

